use std::fmt::{Display, Formatter};
use winit::{
    application::ApplicationHandler, event::WindowEvent, event_loop::ActiveEventLoop, raw_window_handle::{HasDisplayHandle, DisplayHandle, HandleError}, window::{Window, WindowId}
};
use winit::window::WindowAttributes;
use crate::vulkan::VkInstance;

pub struct App {
    window_attributes: WindowAttributes,
    window: Option<Window>,
    instance: Option<VkInstance>,
}

impl App {
    pub fn new(window_attributes: WindowAttributes) -> Self {
        Self {
            window_attributes,
            window: None,
            instance: None,
        }
    }
}

impl HasDisplayHandle for App {   
    fn display_handle(&self) -> Result<DisplayHandle<'_>, HandleError> {
        self.window.as_ref()
            .ok_or_else(|| HandleError::Unavailable)?
            .display_handle()
    }
}

impl ApplicationHandler for App {
    fn resumed(&mut self, event_loop: &ActiveEventLoop) {
        self.window = event_loop
            .create_window(self.window_attributes.clone())
            .ok();

        self.instance = self.window
            .as_ref()
            .and_then(|w| Some(VkInstance::new(w)));

        if let Some(instance) = self.instance.as_ref() {
            let physical_devices = instance.get_physical_devices();

            log::info!("Physical Devices:");
            for physical_device in physical_devices {
                log::info!("{}", physical_device)
            }
        }
    }

    fn window_event(&mut self, event_loop: &ActiveEventLoop, id: WindowId, event: WindowEvent) {
        match event {
            WindowEvent::CloseRequested => {
                log::info!("The close button was pressed; stopping");
                event_loop.exit();
            }
            WindowEvent::RedrawRequested => {
                self.window.as_ref().unwrap().request_redraw();
            }
            _ => (),
        }
    }
}
